@@ -6,6 +6,56 @@ require 'log4r'
 module VagrantPlugins
   module ProviderLibvirt
     module Action
+      action_root = Pathname.new(File.expand_path('../action', __FILE__))
+      autoload :CleanupOnFailure, action_root.join('cleanup_on_failure')
+      autoload :ClearForwardedPorts, action_root.join('forward_ports')
+      autoload :CreateDomain, action_root.join('create_domain')
+      autoload :CreateDomainVolume, action_root.join('create_domain_volume')
+      autoload :CreateNetworkInterfaces, action_root.join('create_network_interfaces')
+      autoload :CreateNetworks, action_root.join('create_networks')
+      autoload :CleanMachineFolder, action_root.join('clean_machine_folder')
+      autoload :DestroyDomain, action_root.join('destroy_domain')
+      autoload :DestroyNetworks, action_root.join('destroy_networks')
+      autoload :ForwardPorts, action_root.join('forward_ports')
+      autoload :HaltDomain, action_root.join('halt_domain')
+      autoload :HandleBoxImage, action_root.join('handle_box_image')
+      autoload :HandleStoragePool, action_root.join('handle_storage_pool')
+      autoload :IsCreated, action_root.join('is_created')
+      autoload :IsRunning, action_root.join('is_running')
+      autoload :IsSuspended, action_root.join('is_suspended')
+      autoload :MessageAlreadyCreated, action_root.join('message_already_created')
+      autoload :MessageNotCreated, action_root.join('message_not_created')
+      autoload :MessageNotRunning, action_root.join('message_not_running')
+      autoload :MessageNotSuspended, action_root.join('message_not_suspended')
+      autoload :MessageWillNotDestroy, action_root.join('message_will_not_destroy')
+      autoload :PackageDomain, action_root.join('package_domain')
+      autoload :PrepareNFSSettings, action_root.join('prepare_nfs_settings')
+      autoload :PrepareNFSValidIds, action_root.join('prepare_nfs_valid_ids')
+      autoload :PruneNFSExports, action_root.join('prune_nfs_exports')
+      autoload :ReadMacAddresses, action_root.join('read_mac_addresses')
+      autoload :RemoveLibvirtImage, action_root.join('remove_libvirt_image')
+      autoload :RemoveStaleVolume, action_root.join('remove_stale_volume')
+      autoload :ResolveDiskSettings, action_root.join('resolve_disk_settings')
+      autoload :ResumeDomain, action_root.join('resume_domain')
+      autoload :SetNameOfDomain, action_root.join('set_name_of_domain')
+      autoload :SetBootOrder, action_root.join('set_boot_order')
+      autoload :SetupComplete, action_root.join('cleanup_on_failure')
+      # I don't think we need it anymore
+      autoload :ShareFolders, action_root.join('share_folders')
+      autoload :ShutdownDomain, action_root.join('shutdown_domain')
+      autoload :StartDomain, action_root.join('start_domain')
+      autoload :StartShutdownTimer, action_root.join('shutdown_domain')
+      autoload :SuspendDomain, action_root.join('suspend_domain')
+      autoload :TimedProvision, action_root.join('timed_provision')
+      autoload :WaitTillUp, action_root.join('wait_till_up')
+
+      autoload :Package, 'vagrant/action/general/package'
+      autoload :PackageSetupFiles, 'vagrant/action/general/package_setup_files'
+      autoload :PackageSetupFolders, 'vagrant/action/general/package_setup_folders'
+      autoload :ProvisionerCleanup, 'vagrant/action/builtin/provisioner_cleanup'
+      autoload :SSHRun, 'vagrant/action/builtin/ssh_run'
+      autoload :SyncedFolderCleanup, 'vagrant/action/builtin/synced_folder_cleanup'
+
       # Include the built-in & general modules so we can use them as top-level things.
       include Vagrant::Action::Builtin
       include Vagrant::Action::General
@@ -24,44 +74,46 @@ module VagrantPlugins
           b.use ConfigValidate
           b.use BoxCheckOutdated
           b.use Call, IsCreated do |env, b2|
+            b2.use CleanupOnFailure
+            b2.use Provision
+
             # Create VM if not yet created.
             if !env[:result]
               b2.use SetNameOfDomain
+
               if !env[:machine].config.vm.box
+                b2.use ResolveDiskSettings
                 b2.use CreateDomain
                 b2.use CreateNetworks
                 b2.use CreateNetworkInterfaces
-                b2.use SetBootOrder
-                b2.use StartDomain
+
+                b2.use action_start
+
+                b2.use SetupComplete
               else
                 b2.use HandleStoragePool
+                require 'vagrant/action/builtin/handle_box'
                 b2.use HandleBox
                 b2.use HandleBoxImage
                 b2.use CreateDomainVolume
+                b2.use ResolveDiskSettings
                 b2.use CreateDomain
-
-                b2.use Provision
-                b2.use PrepareNFSValidIds
-                b2.use SyncedFolderCleanup
-                b2.use SyncedFolders
-                b2.use PrepareNFSSettings
-                b2.use ShareFolders
                 b2.use CreateNetworks
                 b2.use CreateNetworkInterfaces
-                b2.use SetBootOrder
 
-                b2.use StartDomain
-                b2.use WaitTillUp
-                b2.use WaitForCommunicator, [:running]
+                b2.use action_start
 
-                b2.use ForwardPorts
                 b2.use SetHostname
-                # b2.use SyncFolders
               end
             else
               env[:halt_on_error] = true
+              b2.use ResolveDiskSettings
+              b2.use CreateNetworks
               b2.use action_start
             end
+
+            # corresponding action to CleanupOnFailure
+            b2.use SetupComplete
           end
         end
       end
@@ -69,7 +121,7 @@ module VagrantPlugins
       # Assuming VM is created, just start it. This action is not called
       # directly by any subcommand. VM can be suspended, already running or in
       # poweroff state.
-      def self.action_start
+      private_class_method def self.action_start
         Vagrant::Action::Builder.new.tap do |b|
           b.use ConfigValidate
           b.use Call, IsRunning do |env, b2|
@@ -82,28 +134,24 @@ module VagrantPlugins
             b2.use Call, IsSuspended do |env2, b3|
               # if vm is suspended resume it then exit
               if env2[:result]
-                b3.use CreateNetworks
                 b3.use ResumeDomain
                 next
               end
 
               if !env[:machine].config.vm.box
                 # With no box, we just care about network creation and starting it
-                b3.use CreateNetworks
                 b3.use SetBootOrder
                 b3.use StartDomain
               else
                 # VM is not running or suspended.
-
-                b3.use Provision
-
-                # Ensure networks are created and active
-                b3.use CreateNetworks
-                b3.use SetBootOrder
-
                 b3.use PrepareNFSValidIds
                 b3.use SyncedFolderCleanup
+                require 'vagrant/action/builtin/synced_folders'
                 b3.use SyncedFolders
+                b3.use PrepareNFSSettings
+                b3.use ShareFolders
+
+                b3.use SetBootOrder
 
                 # Start it..
                 b3.use StartDomain
@@ -111,11 +159,10 @@ module VagrantPlugins
                 # Machine should gain IP address when comming up,
                 # so wait for dhcp lease and store IP into machines data_dir.
                 b3.use WaitTillUp
+                require 'vagrant/action/builtin/wait_for_communicator'
                 b3.use WaitForCommunicator, [:running]
 
                 b3.use ForwardPorts
-                b3.use PrepareNFSSettings
-                b3.use ShareFolders
               end
             end
           end
@@ -170,6 +217,7 @@ module VagrantPlugins
             end
 
             b2.use ConfigValidate
+            b2.use Provision
             b2.use action_halt
             b2.use action_start
           end
@@ -264,7 +312,6 @@ module VagrantPlugins
               end
 
               b3.use Provision
-              # b3.use SyncFolders
             end
           end
         end
@@ -310,6 +357,9 @@ module VagrantPlugins
               end
               b3.use CreateNetworks
               b3.use ResumeDomain
+              b3.use Provision
+              require 'vagrant/action/builtin/wait_for_communicator'
+              b3.use WaitForCommunicator, [:running]
             end
           end
         end
@@ -342,61 +392,6 @@ module VagrantPlugins
         end
       end
 
-      action_root = Pathname.new(File.expand_path('../action', __FILE__))
-      autoload :PackageDomain, action_root.join('package_domain')
-      autoload :CreateDomain, action_root.join('create_domain')
-      autoload :CreateDomainVolume, action_root.join('create_domain_volume')
-      autoload :CreateNetworkInterfaces, action_root.join('create_network_interfaces')
-      autoload :CreateNetworks, action_root.join('create_networks')
-      autoload :CleanMachineFolder, action_root.join('clean_machine_folder')
-      autoload :DestroyDomain, action_root.join('destroy_domain')
-      autoload :DestroyNetworks, action_root.join('destroy_networks')
-      autoload :ForwardPorts, action_root.join('forward_ports')
-      autoload :ClearForwardedPorts, action_root.join('forward_ports')
-      autoload :HaltDomain, action_root.join('halt_domain')
-      autoload :StartShutdownTimer, action_root.join('shutdown_domain')
-      autoload :ShutdownDomain, action_root.join('shutdown_domain')
-      autoload :HandleBoxImage, action_root.join('handle_box_image')
-      autoload :HandleStoragePool, action_root.join('handle_storage_pool')
-      autoload :RemoveLibvirtImage, action_root.join('remove_libvirt_image')
-      autoload :IsCreated, action_root.join('is_created')
-      autoload :IsRunning, action_root.join('is_running')
-      autoload :IsSuspended, action_root.join('is_suspended')
-      autoload :MessageAlreadyCreated, action_root.join('message_already_created')
-      autoload :MessageNotCreated, action_root.join('message_not_created')
-      autoload :MessageNotRunning, action_root.join('message_not_running')
-      autoload :MessageNotSuspended, action_root.join('message_not_suspended')
-      autoload :MessageWillNotDestroy, action_root.join('message_will_not_destroy')
-
-      autoload :RemoveStaleVolume, action_root.join('remove_stale_volume')
-
-      autoload :PrepareNFSSettings, action_root.join('prepare_nfs_settings')
-      autoload :PrepareNFSValidIds, action_root.join('prepare_nfs_valid_ids')
-      autoload :PruneNFSExports, action_root.join('prune_nfs_exports')
-
-      autoload :ReadMacAddresses, action_root.join('read_mac_addresses')
-      autoload :ResumeDomain, action_root.join('resume_domain')
-      autoload :SetNameOfDomain, action_root.join('set_name_of_domain')
-      autoload :SetBootOrder, action_root.join('set_boot_order')
-
-      # I don't think we need it anymore
-      autoload :ShareFolders, action_root.join('share_folders')
-      autoload :StartDomain, action_root.join('start_domain')
-      autoload :SuspendDomain, action_root.join('suspend_domain')
-      autoload :TimedProvision, action_root.join('timed_provision')
-
-      autoload :WaitTillUp, action_root.join('wait_till_up')
-      autoload :PrepareNFSValidIds, action_root.join('prepare_nfs_valid_ids')
-
-      autoload :Package, 'vagrant/action/general/package'
-      autoload :PackageSetupFiles, 'vagrant/action/general/package_setup_files'
-      autoload :PackageSetupFolders, 'vagrant/action/general/package_setup_folders'
-      autoload :SSHRun, 'vagrant/action/builtin/ssh_run'
-      autoload :HandleBox, 'vagrant/action/builtin/handle_box'
-      autoload :SyncedFolders, 'vagrant/action/builtin/synced_folders'
-      autoload :SyncedFolderCleanup, 'vagrant/action/builtin/synced_folder_cleanup'
-      autoload :ProvisionerCleanup, 'vagrant/action/builtin/provisioner_cleanup'
-      autoload :WaitForCommunicator, 'vagrant/action/builtin/wait_for_communicator'
     end
   end
 end
