@@ -4,15 +4,8 @@ require 'cgi'
 
 require 'vagrant'
 
-class Numeric
-  Alphabet = ('a'..'z').to_a
-  def vdev
-    s = String.new
-    q = self
-    (q, r = (q - 1).divmod(26)) && s.prepend(Alphabet[r]) until q.zero?
-    "vd#{s}"
-  end
-end
+require 'vagrant-libvirt/errors'
+require 'vagrant-libvirt/util/resolvers'
 
 module VagrantPlugins
   module ProviderLibvirt
@@ -69,6 +62,7 @@ module VagrantPlugins
       attr_accessor :management_network_pci_slot
       attr_accessor :management_network_domain
       attr_accessor :management_network_mtu
+      attr_accessor :management_network_keep
 
       # System connection information
       attr_accessor :system_uri
@@ -227,6 +221,7 @@ module VagrantPlugins
         @management_network_pci_bus = UNSET_VALUE
         @management_network_domain = UNSET_VALUE
         @management_network_mtu = UNSET_VALUE
+        @management_network_keep = UNSET_VALUE
 
         # System connection information
         @system_uri      = UNSET_VALUE
@@ -346,24 +341,13 @@ module VagrantPlugins
         # Use Qemu agent to get ip address
         @qemu_use_agent  = UNSET_VALUE
 
-        @serials           = []
+        @serials           = UNSET_VALUE
       end
 
       def boot(device)
         @boot_order << device # append
       end
 
-      def _get_device(disks)
-        # skip existing devices and also the first one (vda)
-        exist = disks.collect { |x| x[:device] } + [1.vdev.to_s]
-        skip = 1 # we're 1 based, not 0 based...
-        loop do
-          dev = skip.vdev # get lettered device
-          return dev unless exist.include?(dev)
-          skip += 1
-        end
-      end
-
       def _get_cdrom_dev(cdroms)
         exist = Hash[cdroms.collect { |x| [x[:dev], true] }]
         # hda - hdc
@@ -516,7 +500,9 @@ module VagrantPlugins
                        target_address: options[:target_address],
                        target_name: options[:target_name],
                        target_port: options[:target_port],
-                       target_type: options[:target_type])
+                       target_type: options[:target_type],
+                       disabled: options[:disabled],
+                      )
       end
 
       def random(options = {})
@@ -542,10 +528,14 @@ module VagrantPlugins
           pci_domain = options[:domain]
         end
 
-        @pcis.push(domain:    pci_domain,
-                   bus:       options[:bus],
-                   slot:      options[:slot],
-                   function:  options[:function])
+        @pcis.push(domain:          pci_domain,
+                   bus:             options[:bus],
+                   slot:            options[:slot],
+                   function:        options[:function],
+                   guest_domain:    options[:guest_domain],
+                   guest_bus:       options[:guest_bus],
+                   guest_slot:      options[:guest_slot],
+                   guest_function:  options[:guest_function])
       end
 
       def watchdog(options = {})
@@ -715,6 +705,8 @@ module VagrantPlugins
       end
 
       def serial(options={})
+        @serials = [] if @serials == UNSET_VALUE
+
         options = {
           :type => "pty",
           :source => nil,
@@ -840,6 +832,7 @@ module VagrantPlugins
         @management_network_pci_slot = nil if @management_network_pci_slot == UNSET_VALUE
         @management_network_domain = nil if @management_network_domain == UNSET_VALUE
         @management_network_mtu = nil if @management_network_mtu == UNSET_VALUE
+        @management_network_keep = false if @management_network_keep == UNSET_VALUE
 
         # Domain specific settings.
         @title = '' if @title == UNSET_VALUE
@@ -892,7 +885,7 @@ module VagrantPlugins
         @graphics_port = -1 if @graphics_port == UNSET_VALUE
         @graphics_ip = '127.0.0.1' if @graphics_ip == UNSET_VALUE
         @video_type = 'cirrus' if @video_type == UNSET_VALUE
-        @video_vram = 9216 if @video_vram == UNSET_VALUE
+        @video_vram = 16384 if @video_vram == UNSET_VALUE
         @video_accel3d = false if @video_accel3d == UNSET_VALUE
         @graphics_gl = @video_accel3d if @graphics_gl == UNSET_VALUE
         @sound_type = nil if @sound_type == UNSET_VALUE
@@ -914,10 +907,6 @@ module VagrantPlugins
 
         # Storage
         @disks = [] if @disks == UNSET_VALUE
-        @disks.map! do |disk|
-          disk[:device] = _get_device(@disks) if disk[:device].nil?
-          disk
-        end
         @cdroms = [] if @cdroms == UNSET_VALUE
         @cdroms.map! do |cdrom|
           cdrom[:dev] = _get_cdrom_dev(@cdroms) if cdrom[:dev].nil?
@@ -928,7 +917,18 @@ module VagrantPlugins
         @inputs = [{ type: 'mouse', bus: 'ps2' }] if @inputs == UNSET_VALUE
 
         # Channels
-        @channels = [] if @channels == UNSET_VALUE
+        if @channels == UNSET_VALUE
+          @channels = []
+          if @qemu_use_agent == true
+            if @channels.all? { |channel| !channel.fetch(:target_name, '').start_with?('org.qemu.guest_agent.') }
+              channel(:type => 'unix', :target_name => 'org.qemu.guest_agent.0', :target_type => 'virtio')
+            end
+          end
+        end
+
+        # filter channels of anything explicitly disabled so it's possible to inject an entry to
+        # avoid the automatic addition of the guest_agent above, and disable it from subsequent use.
+        @channels = @channels.reject { |channel| channel[:disabled] }.tap {|channel| channel.delete(:disabled) }
 
         # PCI device passthrough
         @pcis = [] if @pcis == UNSET_VALUE
@@ -971,12 +971,17 @@ module VagrantPlugins
 
         @qemu_use_agent = false if @qemu_use_agent == UNSET_VALUE
 
-        @serials = [{:type => 'pty', :source => nil}] if @serials == []
+        @serials = [{:type => 'pty', :source => nil}] if @serials == UNSET_VALUE
       end
 
       def validate(machine)
         errors = _detected_errors
 
+        # technically this shouldn't occur, but ensure that if somehow it does, it gets rejected.
+        if @cpu_mode == 'host-passthrough' && @cpu_model != ''
+          errors << "cannot set cpu_model with cpu_mode of 'host-passthrough'. leave model unset or switch mode."
+        end
+
         # The @uri and @qemu_use_session should not conflict
         uri = _parse_uri(@uri)
         if (uri.scheme.start_with? "qemu") && (uri.path.include? "session")
@@ -989,6 +994,10 @@ module VagrantPlugins
           errors << "libvirt.qemu_use_agent must be a boolean."
         end
 
+        if !@nvram.nil? && @loader.nil?
+          errors << "use of 'nvram' requires a 'loader' to be specified, please add one to the configuration"
+        end
+
         if @qemu_use_agent == true
           # if qemu agent is used to optain domain ip configuration, at least
           # one qemu channel has to be configured. As there are various options,
@@ -998,7 +1007,6 @@ module VagrantPlugins
           end
         end
 
-
         machine.provider_config.disks.each do |disk|
           if disk[:path] && (disk[:path][0] == '/')
             errors << "absolute volume paths like '#{disk[:path]}' not yet supported"
@@ -1011,6 +1019,16 @@ module VagrantPlugins
           end
         end
 
+        # this won't be able to fully resolve the disks until the box has
+        # been downloaded and any devices that need to be assigned to the
+        # disks contained have been allocated
+        disk_resolver = ::VagrantPlugins::ProviderLibvirt::Util::DiskDeviceResolver.new
+        begin
+          disk_resolver.resolve(machine.provider_config.disks)
+        rescue Errors::VagrantLibvirtError => e
+          errors << "#{e}"
+        end
+
         machine.config.vm.networks.each do |_type, opts|
           if opts[:mac]
             if opts[:mac] =~ /\A([0-9a-fA-F]{12})\z/
@@ -1053,9 +1071,11 @@ module VagrantPlugins
           c.merge!(other.qemu_env) if other.qemu_env != UNSET_VALUE
           result.qemu_env = c
 
-          s = serials.dup
-          s += other.serials
-          result.serials = s
+          if serials != UNSET_VALUE
+            s = serials.dup
+            s += other.serials
+            result.serials = s
+          end
         end
       end
 
