@@ -32,6 +32,7 @@ module VagrantPlugins
           management_network_pci_slot = env[:machine].provider_config.management_network_pci_slot
           management_network_domain = env[:machine].provider_config.management_network_domain
           management_network_mtu = env[:machine].provider_config.management_network_mtu
+          management_network_keep = env[:machine].provider_config.management_network_keep
           logger.info "Using #{management_network_name} at #{management_network_address} as the management network #{management_network_mode} is the mode"
 
           begin
@@ -92,8 +93,18 @@ module VagrantPlugins
             management_network_options[:slot] = management_network_pci_slot
           end
 
-          if (env[:machine].config.vm.box &&
-              !env[:machine].provider_config.mgmt_attach)
+          if management_network_keep
+            management_network_options[:always_destroy] = false
+          end
+
+          # if there is a box and management network is disabled
+          # need qemu agent enabled and at least one network that can be accessed
+          if (
+            env[:machine].config.vm.box &&
+            !env[:machine].provider_config.mgmt_attach &&
+            !env[:machine].provider_config.qemu_use_agent &&
+            !env[:machine].config.vm.networks.any? { |type, _| ["private_network", "public_network"].include?(type.to_s) }
+          )
             raise Errors::ManagementNetworkRequired
           end
 
