@@ -15,7 +15,6 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
   include_context 'unit'
   include_context 'libvirt'
 
-  let(:libvirt_client) { double('libvirt_client') }
   let(:volumes) { double('volumes') }
   let(:all) { double('all') }
   let(:box_volume) { double('box_volume') }
@@ -54,17 +53,13 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
   byte_number_20G = ByteNumber.new(21474836480)
 
 
-
-
   describe '#call' do
     before do
-      allow_any_instance_of(VagrantPlugins::ProviderLibvirt::Driver)
-        .to receive(:connection).and_return(connection)
-      allow(connection).to receive(:client).and_return(libvirt_client)
       allow(connection).to receive(:volumes).and_return(volumes)
       allow(volumes).to receive(:all).and_return(all)
       allow(env[:ui]).to receive(:clear_line)
 
+      env[:machine].provider_config.disk_device = 'vda'
     end
 
     context 'when one disk in metadata.json' do
@@ -92,7 +87,8 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
               :path=>"/test/box.img",
               :name=>"test_vagrant_box_image_1.1.1_box.img",
               :virtual_size=>byte_number_5G,
-              :format=>"qcow2"
+              :format=>"qcow2",
+              :device=>'vda',
             }
           ]
         )
@@ -121,7 +117,8 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
                 :path=>"/test/box.img",
                 :name=>"test_vagrant_box_image_0_#{box_mtime.to_i}_box.img",
                 :virtual_size=>byte_number_5G,
-                :format=>"qcow2"
+                :format=>"qcow2",
+                :device=>'vda',
               }
             ]
           )
@@ -130,7 +127,7 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
 
       context 'When config.machine_virtual_size is set and smaller than box_virtual_size' do
         before do
-          allow(env[:machine]).to receive_message_chain("provider_config.machine_virtual_size").and_return(1)
+          env[:machine].provider_config.machine_virtual_size = 1
         end
         it 'should warning must be raise' do
           expect(ui).to receive(:warn).with("Ignoring requested virtual disk size of '1' as it is below\nthe minimum box image size of '5'.")
@@ -141,7 +138,8 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
                 :path=>"/test/box.img",
                 :name=>"test_vagrant_box_image_1.1.1_box.img",
                 :virtual_size=>byte_number_5G,
-                :format=>"qcow2"
+                :format=>"qcow2",
+                :device=>'vda',
               }
             ]
           )
@@ -150,7 +148,7 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
 
       context 'When config.machine_virtual_size is set and higher than box_virtual_size' do
         before do
-          allow(env[:machine]).to receive_message_chain("provider_config.machine_virtual_size").and_return(20)
+          env[:machine].provider_config.machine_virtual_size = 20
         end
         it 'should be use' do
           expect(ui).to receive(:info).with("Created volume larger than box defaults, will require manual resizing of\nfilesystems to utilize.")
@@ -161,7 +159,8 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
                 :path=>"/test/box.img",
                 :name=>"test_vagrant_box_image_1.1.1_box.img",
                 :virtual_size=>byte_number_20G,
-                :format=>"qcow2"
+                :format=>"qcow2",
+                :device=>'vda',
               }
             ]
           )
@@ -252,7 +251,8 @@ describe VagrantPlugins::ProviderLibvirt::Action::HandleBoxImage do
               :path=>"/test/box.img",
               :name=>"test_vagrant_box_image_1.1.1_send_box_name.img",
               :virtual_size=>byte_number_5G,
-              :format=>"qcow2"
+              :format=>"qcow2",
+              :device=>'vda',
             },
             {
               :path=>"/test/disk.qcow2",
