@@ -59,16 +59,16 @@ describe 'templates/domain' do
       domain.disk_device = 'vda'
       domain.disk_driver(:cache => 'unsafe', :io => 'threads', :copy_on_read => 'on', :discard => 'unmap', :detect_zeroes => 'on')
       domain.domain_volumes.push({
-        :dev => 1.vdev.to_s,
+        :dev => 'vda',
         :cache => 'unsafe',
         :bus => domain.disk_bus,
-        :path => '/var/lib/libvirt/images/test.qcow2'
+        :absolute_path => '/var/lib/libvirt/images/test.qcow2'
       })
       domain.domain_volumes.push({
-        :dev => 2.vdev.to_s,
+        :dev => 'vdb',
         :cache => 'unsafe',
         :bus => domain.disk_bus,
-        :path => '/var/lib/libvirt/images/test2.qcow2'
+        :absolute_path => '/var/lib/libvirt/images/test2.qcow2'
       })
       domain.storage(:file, path: 'test-disk1.qcow2')
       domain.storage(:file, path: 'test-disk2.qcow2', io: 'threads', copy_on_read: 'on', discard: 'unmap', detect_zeroes: 'on')
@@ -92,6 +92,7 @@ describe 'templates/domain' do
       domain.serial(:type => 'file', :source => {:path => '/var/log/vm_consoles/machine.log'})
       domain.pci(bus: '0x06', slot: '0x12', function: '0x5')
       domain.pci(domain: '0x0001', bus: '0x03', slot: '0x00', function: '0x0')
+      domain.pci(domain: '0x0002', bus: '0x04', slot: '0x00', function: '0x0', guest_domain: '0x0000', guest_bus: '0x01', guest_slot: '0x01', guest_function: '0x0')
       domain.usb_controller(model: 'nec-xhci', ports: '4')
       domain.usb(bus: '1', device: '2', vendor: '0x1234', product: '0xabcd')
       domain.redirdev(type: 'tcp', host: 'localhost', port: '4000')
@@ -118,19 +119,42 @@ describe 'templates/domain' do
     let(:test_file) { 'domain_all_settings.xml' }
     it 'renders template' do
       domain.finalize!
+      # resolving is now done during create domain, so need to recreate
+      # the same behaviour before calling the template until that
+      # is separated out from create domain.
+      resolver = ::VagrantPlugins::ProviderLibvirt::Util::DiskDeviceResolver.new(prefix=domain.disk_device[0..1])
+      resolver.resolve!(domain.domain_volumes.dup.each { |volume| volume[:device] = volume[:dev] })
+      resolver.resolve!(domain.disks)
+
       expect(domain.to_xml('domain')).to eq xml_expected
     end
   end
 
-  context 'when custom cpu model enabled' do
-    before do
-      domain.cpu_mode = 'custom'
-      domain.cpu_model = 'SandyBridge'
+  context 'when cpu mode is set' do
+    context 'to host-passthrough' do
+      before do
+        domain.cpu_mode = 'host-passthrough'
+        domain.cpu_model = 'SandyBridge'
+        domain.cputopology :sockets => '1', :cores => '2', :threads => '1'
+        domain.nested = true
+      end
+      let(:test_file) { 'domain_cpu_mode_passthrough.xml' }
+      it 'should allow features and topology and ignore model' do
+        domain.finalize!
+        expect(domain.to_xml('domain')).to eq xml_expected
+      end
     end
-    let(:test_file) { 'domain_custom_cpu_model.xml' }
-    it 'renders template' do
-      domain.finalize!
-      expect(domain.to_xml('domain')).to eq xml_expected
+
+    context 'to custom and model is set' do
+      before do
+        domain.cpu_mode = 'custom'
+        domain.cpu_model = 'SandyBridge'
+      end
+      let(:test_file) { 'domain_custom_cpu_model.xml' }
+      it 'renders template' do
+        domain.finalize!
+        expect(domain.to_xml('domain')).to eq xml_expected
+      end
     end
   end
 
@@ -167,12 +191,12 @@ describe 'templates/domain' do
       end
     end
 
-    context 'memballon enabled' do
+    context 'memballoon enabled' do
       before do
         domain.memballoon_enabled = true
       end
 
-      it 'renders with memballon element' do
+      it 'renders with memballoon element' do
         domain.finalize!
 
         expect(domain.to_xml('domain')).to match(/<memballoon model='virtio'>/)
@@ -195,7 +219,7 @@ describe 'templates/domain' do
       end
     end
 
-    context 'memballon disabled' do
+    context 'memballoon disabled' do
       before do
         domain.memballoon_enabled = false
       end
